unit AfWebBrowserImpl;

{$WARN SYMBOL_PLATFORM OFF}

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  ActiveX, AxCtrls, OleObjectsForSasAf_TLB, StdVcl, OleCtrls, SHDocVw;

type
  TMyActiveFormFactory = class (AxCtrls.TActiveFormFactory)
  public
    procedure UpdateRegistry(Register:Boolean);override;
  end;

type
  TAfWebBrowser = class(TActiveForm, IAfWebBrowser)
    WebBrowser1: TWebBrowser;
    procedure ActiveFormCreate(Sender: TObject);
    procedure OnBeforeNavigate2(ASender: TObject;
      const pDisp: IDispatch; var URL, Flags, TargetFrameName, PostData,
      Headers: OleVariant; var Cancel: WordBool);
    procedure ActiveFormDestroy(Sender: TObject);
  private
    { Private declarations }
    FEvents: IAfWebBrowserEvents;
    GetParameters: TStringList;
    PostParameters: TStringList;
  protected
    procedure SplitParams(const ParamString: String; ParamList: TStrings);
    procedure EventSinkChanged(const EventSink: IUnknown); override;
    function Get_URL: OleVariant; safecall;
    procedure Set_URL(Value: OleVariant); safecall;
    function Get_GetCount: Integer; safecall;
    function Get_PostCount: Integer; safecall;
    function Get_GetName(Index: Integer): OleVariant; safecall;
    function Get_GetValue(Index: Integer): OleVariant; safecall;
    function Get_PostName(Index: Integer): OleVariant; safecall;
    function Get_PostValue(Index: Integer): OleVariant; safecall;
  public
    { Public declarations }
    procedure Initialize; override;
  end;

implementation

uses ComObj, ComServ, Variants;

{$R *.DFM}

{ TAfWebBrowser }

procedure TAfWebBrowser.ActiveFormDestroy(Sender: TObject);
begin
  GetParameters.Free;
  PostParameters.Free;
end;

procedure TAfWebBrowser.EventSinkChanged(const EventSink: IUnknown);
begin
  FEvents := EventSink as IAfWebBrowserEvents;
  inherited EventSinkChanged(EventSink);
end;

procedure TAfWebBrowser.Initialize;
begin
  inherited Initialize;
  // On{ActiveXEvent} := {ActiveXEvent}Event;
end;

procedure TAfWebBrowser.ActiveFormCreate(Sender: TObject);
begin
  GetParameters := TStringList.Create;
  PostParameters := TStringList.Create;
end;


function TAfWebBrowser.Get_URL: OleVariant;
begin
  Result := WebBrowser1.LocationURL;
end;

procedure TAfWebBrowser.Set_URL(Value: OleVariant);
begin
  GetParameters.Clear;
  PostParameters.Clear;
  WebBrowser1.Navigate(Value);
end;


function UrlDecode(const EncodedStr: String): String;
var
  I, H1, H2: Integer;
begin
  Result := '';
  if Length(EncodedStr) > 0 then
  begin
    I := 1;
    while I <= Length(EncodedStr) do
    begin
      if (EncodedStr[I] = '%')
      and (I < Length(EncodedStr)-1)
      and (Upcase(EncodedStr[I+1]) in ['0'..'9', 'A'..'F'])
      and (Upcase(EncodedStr[I+2]) in ['0'..'9', 'A'..'F'])
      then
      begin
        H1 := Byte(Upcase(EncodedStr[I+1])) - 48;
        if H1 > 9 then Dec(H1,7);
        H2 := Byte(Upcase(EncodedStr[I+2])) - 48;
        if H2 > 9 then Dec(H2,7);

        Result := Result + Chr(H1 shl 4 + H2);
        I := Succ(Succ(I));
      end
      else if EncodedStr[I] = '+' then
        Result := Result + ' '
      else
        Result := Result + EncodedStr[I];

      I := Succ(I);
    end;
  end;
end;

procedure TAfWebBrowser.SplitParams(const ParamString: String; ParamList: TStrings);
var
  p: Integer;
  Remnant: String;
  Parameter: String;
begin
  if not Assigned(ParamList) then Exit;

  ParamList.Clear;

  if ParamString = '' then Exit;

  Remnant := ParamString;
  p := Pos('&', Remnant);

  while (p > 0) do
  begin
    Parameter := Copy(Remnant, 1, p -1);
    ParamList.Add(UrlDecode(Parameter));
    Remnant := Copy(Remnant, P+1, Length(Remnant) - p);
    p := Pos('&', Remnant);
  end;
  Parameter := Remnant;
  ParamList.Add(UrlDecode(Parameter));
end;

procedure TAfWebBrowser.OnBeforeNavigate2(ASender: TObject;
  const pDisp: IDispatch; var URL, Flags, TargetFrameName, PostData,
  Headers: OleVariant; var Cancel: WordBool);
var
  QueryPos: integer;
  V: OleVariant;
  PostString: String;
begin
  QueryPos := Pos('?', URL);
  if QueryPos > 0 then begin
    SplitParams(Copy(URL, QueryPos+1, Length(URL)-QueryPos), GetParameters);
  end;

  V := Variant(TVarData(PostData).VPointer^);
  if VarType(V) = (varArray + varByte)
  then PostString := PChar(TVarData(V).vArray.Data)
  else PostString := '';

  SplitParams(PostString, PostParameters);

  if FEvents <> nil then FEvents.OnBeforeNavigate;
end;


function TAfWebBrowser.Get_GetCount: Integer;
begin
  Result := GetParameters.Count;
end;

function TAfWebBrowser.Get_PostCount: Integer;
begin
  Result := PostParameters.Count;
end;

function TAfWebBrowser.Get_GetName(Index: Integer): OleVariant;
var p: integer;
begin
  p := Pos('=',GetParameters[index]);
  if p > 1 then
    Result := Copy(GetParameters[index],1,p-1)
  else
    Result := '';
end;

function TAfWebBrowser.Get_GetValue(Index: Integer): OleVariant;
var p: integer;
begin
  p := Pos('=',GetParameters[index]);
  Result := Copy(GetParameters[index],p+1, Length(GetParameters[index])-p);
end;

function TAfWebBrowser.Get_PostName(Index: Integer): OleVariant;
var p: integer;
begin
  p := Pos('=',PostParameters[index]);
  if p > 1 then
    Result := Copy(PostParameters[index],1,p-1)
  else
    Result := '';
end;

function TAfWebBrowser.Get_PostValue(Index: Integer): OleVariant;
var p: integer;
begin
  p := Pos('=',PostParameters[index]);
  Result := Copy(PostParameters[index],p+1, Length(PostParameters[index])-p);
end;

{ TMyActiveFormFactory }

procedure TMyActiveFormFactory.UpdateRegistry(Register: Boolean);
var
  ClassKey: string;
begin
  ClassKey := 'CLSID\' + GUIDToString(ClassID) + '\';

  if Register then
  begin
    inherited;
    CreateRegKey(ClassKey + 'Insertable', '', '');
  end
  else begin
    DeleteRegKey(ClassKey + 'Insertable');
    inherited;
  end;
end;

initialization
  TMyActiveFormFactory.Create(
    ComServer,
    TActiveFormControl,
    TAfWebBrowser,
    Class_AfWebBrowser,
    1,
    '',
    OLEMISC_SIMPLEFRAME or OLEMISC_ACTSLIKELABEL,
    tmApartment);
end.
